/* Syllogy: allows table generation and searching using limited NLI */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>

#define MOST 50
#define VRBS 9
#define KEEP 0
#define BLK 1
#define RED 2
#define GRN 3
#define YLO 4
#define BLU 5
#define MAG 6
#define CYN 7
#define WHI 8

char article[3][5];          /* Article Table */
char verb[VRBS][9];          /* Verb Table */
char names[MOST][30];        /* Name list */
int crossref[MOST][MOST];    /* Cross-reference for Name Equalities */
int commandcounter;          /* Aid to keeping the screen clean */

struct sentence {            /* for Command parsing data returns */
    int v1;
    char n1[30];
    char n2[30];
    };

extern int isansiok, isbiosok;

/* ************************* Main Core ********************************* */

main()                       /* This is the heart of the Syllogy Program */
{
    int cntinu;
    struct sentence s;

    setup();                 /* Fill Tables and Blank Out the Lists */

    cntinu = 1;

    while(cntinu == 1) {

         command(&s);         /* Get a command...parse it & return structure */

         switch (s.v1) {
              case 0: statement(&s);
                   break;
              case 1: question(&s);
                   break;
              case 2: inquiry(&s);
                   break;
              case 3: save();
                   break;
              case 4: get();
                   break;
              case 5: help();
                   break;
              case 6: cleanup();
                   break;
              case 7: cntinu = 0;
                   break;
              default:
                   sscs(KEEP, RED, KEEP);
                   printf("Bad Verb Value...fits no case\n");
                   sscs(KEEP, GRN, KEEP);
         }
    }

    sscs(1, WHI, BLK);
    scls();
    printf("\nSyllogy has Terminated Normally...hope you enjoyed...\n");
    printf("If you are interested in our commercial programs, call or write:\n");
    printf("            Thunderhead Software, Inc.\n");
    printf("            P.O. BOX 91\n");
    printf("            McKeesport, Pa. 15135\n");
    printf("            412-751-4020: Rich Parker\n\n");

}

/* *******************Sub Procedures for Master Loop******************* */

statement(sen)          /* This Routine Processes a Statement */
struct sentence *sen;
{
    int noun1, noun2, pointer, pointer2;

    if( ((noun1=pushit(sen->n1)) == -1) || ((noun2=pushit(sen->n2)) == -1) ) {
         sscs(KEEP, RED, KEEP);
         printf("No Room for additional nouns!!!\n");
         sscs(KEEP, GRN, KEEP);
         return(0);
    }

    crossref[noun1][noun2] = 1;

    for( pointer = 0; pointer < MOST; pointer++) {

         if( crossref[noun2][pointer] )
              crossref[noun1][pointer] = 1;

         if( crossref[pointer][noun1] ) {
              for( pointer2 = 0; pointer2 < MOST; pointer2++) {
                   if( crossref[noun2][pointer2] )
                        crossref[pointer][pointer2] = 1;
              }
         }
    }

    printf("Acknowledged\n");

}

question(sen)           /* This Routine Processes a Question */
struct sentence *sen;
{
    int noun1, noun2;

    if( (noun1 = findit(sen->n1)) == -1 ) {
         sscs(KEEP, RED, KEEP);
         printf("I don't recognize %s\n", sen->n1);
    }

    if( (noun2 = findit(sen->n2)) == -1 ) {
         sscs(KEEP, RED, KEEP);
         printf("I don't recognize %s\n", sen->n2);
    }

    sscs(KEEP, GRN, KEEP);

    if( (noun1 == -1) || (noun2 == -1) )
         return(0);

    if( crossref[noun1][noun2] == 1 )
         printf("                                  YES \n");
    else
         printf("                                   NO \n");
}

inquiry(sen)                 /* This Routine Processes an Inquiry */
struct sentence *sen;
{
    int noun1, pointer;

    if( (noun1 = findit(sen->n1)) == -1 ) {
         sscs(KEEP, RED, KEEP);
         printf("I do not recognize %s\n", sen->n1);
         sscs(KEEP, GRN, KEEP);
         return(0);
    }
    clear();
    printf("\nResults of inquiry on: %s %d\n", sen->n1, noun1);

    for( pointer = 0; pointer < MOST; pointer++) {
         if( crossref[noun1][pointer] == 1 ) 
              printf(" %s\n", names[pointer]);
    }
}

help()
{
    int pointer, pointer2;
    char *entered;

    entered = malloc(30);

    clear();
    sscs(KEEP, YLO, KEEP);
    printf("-----help:\n");
    printf("\nArticles: %s, %s, %s\n", article[0], article[1], article[2]);
    printf("\nVerbs: ");
    for(pointer = 0; pointer < VRBS; pointer++)
         printf("%s ", verb[pointer]);
    pointer2 = 7;
    printf("\n\n\nNouns: ");
    for(pointer = 0; isalpha(*names[pointer]); pointer++) {
         printf("%s  ", names[pointer]);
         pointer2 += (strlen(names[pointer]) + 2);
         if( pointer2 > 60) {
              printf("\n       ");
              pointer2 = 7;
         }
    }
    printf("\n\n\nPress <Enter> To Continue:");
    gets(entered);
    free(entered);
    clear();
}

/* ..............Sub Procedures for above Procedures ........... */

pushit(object)
char *object;
{
    int pointer;

    if( (pointer = findit(object)) > -1 )
         return(pointer);

    for( pointer = 0; pointer < MOST; pointer++) {
         if( !(isalpha(*names[pointer])) ) {
              strcpy(names[pointer], object);
              crossref[pointer][pointer] = 1;
              return(pointer);
         }
    }

    return(-1);
}

findit(object)
char *object;
{
    int pointer;
    for( pointer = 0; pointer < MOST; pointer++) {

         if( !(strcmp(object, names[pointer])) )
              return(pointer);
    }

    return(-1);
}

/* --------------------- Disk Procedures -----------------------*/

save()             /* This routine saves the Knowledge Base to disk */
{
    FILE *fp;
    char *entered, *entered2;
    int pointer, pointer1;

    entered = malloc(80);
    entered2 = malloc(40);

    clear();
    printf("Save Routine\n");
    sscs(KEEP, BLU, KEEP);
    printf("Enter Filename: ");
    sscs(KEEP, WHI, KEEP);
    gets(entered);
    sscs(KEEP, BLU, KEEP);

    if( fp = fopen(entered, "rb")){
         fclose(fp);
         for(;;) {
              sscs(KEEP, RED, KEEP);
              printf("\nFile Already Exists...Overwrite it (y/n): ");
              sscs(KEEP, WHI, KEEP);
              gets(entered2);
              sscs(KEEP, BLU, KEEP);
              if( !(strcmp(entered2, "n"))) {
                   clear();
                   return(0);
              }
              if( !(strcmp(entered2, "y")))
                   break;
         }
    }

    if( !(fp = fopen(entered, "wb")) ) {
         sscs(KEEP, RED, KEEP);
         printf("File Opening Error - Returning To Main Program");
         sscs(KEEP, GRN, KEEP);
         return(0);
    }

    for(pointer=0; (pointer < MOST) && (isalpha(*names[pointer])); pointer++) ;

    printf("Saving %d elements to %s\n", pointer, entered);
    fwrite(&pointer, sizeof(int), 1, fp);

    for(pointer1 = 0; pointer1 < pointer; pointer1++) {
         fwrite(names[pointer1], sizeof(char), 30, fp);
         fwrite(crossref[pointer1], sizeof(int), pointer, fp);
    }
    fclose(fp);
    printf("Press <Enter> To Continue:");
    gets(entered);
    clear();
    free(entered);
    return(0);
}

get()              /* This routine retrieves a previously saved K. B. */
{

    FILE *fp;
    char *entered;
    int pointer, pointer1;

    entered = malloc(80);

    clear();
    printf("Get Routine\n");
    sscs(KEEP, BLU, KEEP);
    printf("Enter Filename: ");
    sscs(KEEP, WHI, KEEP);
    gets(entered);
    sscs(KEEP, BLU, KEEP);

    if( !(fp = fopen(entered, "rb")) ) {
         sscs(KEEP, RED, KEEP);
         printf("File Opening Error - Returning To Main Program");
         sscs(KEEP, GRN, KEEP);
         return(0);
    }

    cleanup();               /* erase memory before loading new array */

    fread(&pointer, sizeof(int), 1, fp);

    printf("Getting %d elements from %s\n", pointer, entered);

    for(pointer1 = 0; pointer1 < pointer; pointer1++) {
         fread(names[pointer1], sizeof(char), 30, fp);
         fread(crossref[pointer1], sizeof(int), pointer, fp);
    }
    fclose(fp);
    printf("Press <Enter> to Continue:");
    gets(entered);
    clear();
    free(entered);
    return(0);
}

/* ++++++++++++++++++++ The Command Parser +++++++++++++++++++++ */

command(sen)
struct sentence *sen;
{
    char *entered;
    int sentlen, wordcount, nouns, verbs, verbvalue;
    int pointer, wordlen;
    char parse[10][30];
    char noun[2][30];

    entered = malloc(80);

    for(pointer = 0; pointer < 10; pointer++) {
         for(sentlen = 0; sentlen < 30; sentlen++)
              parse[pointer][sentlen] = 0;
    }

    for(pointer = 0; pointer < 2; pointer++) {
         for(sentlen = 0; sentlen < 30; sentlen++)
              noun[pointer][sentlen] = 0;
    }

/*====================== Get a Line =================================*/

    sentlen = 0;

    while(sentlen < 3) {
         printf("Command:");
         sscs(KEEP, WHI, KEEP);
         gets(entered);
         sscs(KEEP, GRN, KEEP);
         sentlen = strlen(entered);
         commandcounter++;
         if(commandcounter > 4)
              clear();
    }

    printf("accepted...\n");

    wordlen = 0;
    wordcount = 0;

/*====================== Load Parse Array ==========================*/

    for( pointer = 0; pointer <= sentlen; pointer++) {

         if( (*(entered+pointer) != 32) && (*(entered+pointer) != 0) ) 
              wordlen++;

         else if(wordlen > 0) {
              strncpy(parse[wordcount], (entered+pointer-wordlen), wordlen);
              wordlen = 0;
              wordcount++;
         }

    }

    if(wordlen > 0)
         strncpy(parse[wordcount], (entered+pointer-wordlen), wordlen);

/*===================== Parse Array Now Loaded ======================*/

/*====================== Declination Starts Here ====================*/

    verbs = 0;
    nouns = 0;
    verbvalue = -1;

    for(pointer = 0; pointer < wordcount; pointer++) {
         if( (!(articlecheck(parse[pointer]))) && (nouns < 2)) {
              if( (verbs = verbcheck(parse[pointer])) == -1) {
                   strcpy( noun[nouns], parse[pointer] );
                   nouns++;
              }
              else if(nouns > 0)
                   verbvalue = 0;
              else
                   verbvalue = verbs;
         }
    }

    sen->v1 = verbvalue;
    strcpy(sen->n1, noun[0]);
    strcpy(sen->n2, noun[1]);

    free(entered);

}

/* ---------------Sub Procedures for Command Parser----------------*/

articlecheck(inval)
char *inval;
{
    int pointr;

    for(pointr = 0; pointr < 3; pointr++) {
         if( !(strcmp(inval, article[pointr])) )
              return(1);
    }

    return(0);
}

verbcheck(inval)
char *inval;
{
    int pointr;

    for(pointr = 0; pointr < VRBS; pointr++) {
         if( !(strcmp(inval, verb[pointr])) ) {
              if(pointr < 3)
                   return(1);
              else
                   return(pointr-1);
         }
    }
    return(-1);
}

setup()       /* sets up base data arrays */
{
    char bull[10];

    commandcounter = 0;

    strcpy(article[0],"a");
    strcpy(article[1],"an");
    strcpy(article[2],"the");

    strcpy(verb[0],"am");
    strcpy(verb[1],"is");
    strcpy(verb[2],"are");
    strcpy(verb[3],"inquire");
    strcpy(verb[4],"save");
    strcpy(verb[5],"get");
    strcpy(verb[6],"help");
    strcpy(verb[7],"forget");
    strcpy(verb[8],"quit");

    printf( "\n\nShould I use ANSI color codes?\n");
    printf( "(does your computer have device=ansi.sys in config.sys on boot disk?)");
    fflush(stdout); gets(bull);
    if( bull[0] == 'y' || bull[0] == 'Y' ) isansiok = 1;
    else {
        isansiok = 0;
        printf( "\n\nMay I use BIOS screen commands?");
        fflush(stdout); gets(bull);
        if( bull[0] == 'y' || bull[0] == 'Y' ) isbiosok = 1;
        else isbiosok = 0;
    }

    cleanup();

    clear();

}

clear()                 /* sets up screen with header */
{
    int pointer;

    sscs(2, GRN, BLU);
    scls();

    printf("       SYLLOGY: copyright (c) 1990 by Richard F. Parker\n");
    for( pointer = 0; pointer < 80; pointer++)
         printf("-");
    printf("\n");

    commandcounter = 0;
}

cleanup()               /* erases contents of master arrays */
{
    int pointer, pointer2;

    for( pointer = 0; pointer < MOST; pointer++) {
         for( pointer2 = 0; pointer2 < MOST; pointer2++)
              crossref[pointer][pointer2] = 0;
         for( pointer2 = 0; pointer2 < 30; pointer2++)
              names[pointer][pointer2] = 0;
    }
}


